function Sigma = taperCov(U, para)


% Input:
%   U -- matrix of n (number of instances) by d(number of dimension) 
%  para -- a structure of parameters: including parameters for Matern
%  covariance functions and choices of tampers
% Output:
%   Sigma -- the tapered sparse matrix 

[n, d] = size(U);

% parameters of 
nu = para.nu;
alpha = para.alpha;
phi = para.phi;
theta = para.theta;
decision = para.decision;

% error check
if strcmp(para.taper,'Spherical') && nu>0.5
    error('For sperical taper function, nu must <=0.5')
end
    
if strcmp(para.taper,'Wendland1') && nu>1.5
    error('For Wendland1 taper function, nu must <=1.5')
end

if strcmp(para.taper,'Wendland2') && nu>2.5
    error('For Wendland2 taper function, nu must <=2.5')
end

%% distance function
switch para.distance
    case 'Euclidean'
        if ~para.savestorage
            P=sum(U.*U,2);
            H = sqrt(abs(repmat(P',n,1) + repmat(P,1,n) - 2*(U*U')));
            Hvec = reshape(H, n*n,1);
        else
        % second choice: to save storage space, only cases of H<theta
            nzmax = n*n;
            Hvec = sparse(nzmax,1);
            Hindi = sparse(nzmax,1);
            Hindj = sparse(nzmax,1);
            curlen = 0;
            for i=1: n
                tmp =  sqrt( sum((repmat(U(i,:),n,1) - U).^2, 2) );
                [indi, indj] = find( tmp< theta);
                len = length(indi);
                Hindi(curlen+1:  curlen+len) = i*ones(len,1);
                Hindj(curlen+1:  curlen+len) = indi;
                Hvec(curlen+1:  curlen+len) = tmp(indi);
                curlen = curlen+ len;
            end
            if curlen < nzmax
                Hindi = full(Hindi(1:curlen,1));
                Hindj = full(Hindj(1:curlen,1));
                Hvec = full(Hvec(1:curlen,1));
            end
        end
%     case 'others'
%         break;
end
Hvec = Hvec.* (abs(Hvec)> decision);
%Hvec = Hvec + decision;

%% automatic define theta
% % theta = 3*(mean(Hvec)+max(Hvec))/4;

%% taper function used
switch para.taper
    case 'Spherical'
        CovTapvec = ((1- Hvec/theta).*(Hvec < theta )).^2 .*  (1 + Hvec/(2*theta));       
    case 'Wendland1'
        CovTapvec = ((1- Hvec/theta).*( Hvec < theta )).^4.*  (1 + 4*Hvec/theta); 
    case 'Wendland2'
        CovTapvec = ((1- Hvec/theta).*( Hvec < theta )).^6.* (1 + 6*Hvec/theta + 35*Hvec.^2/3/(theta.^2) ); 
end
  
%% calculate the tapered covariance function
if ~para.savestorage
    CovTapvec = CovTapvec .* (CovTapvec > decision);
    [ind, indj] = find(CovTapvec); %indj will all be one
    
    t= besselk(nu,alpha*Hvec(ind));%t= besselk(nu,alpha*Hvec(ind)+decision), to avoid singular besselk value
%     Sigmavec(find(isnan(Sigmavec))) = 1;
    Sigmavec = phi/2^(nu-1)/gamma(nu)* (alpha*Hvec(ind)).^nu.* ...
        t.* CovTapvec(ind);
    Sigmavec(find(isnan(Sigmavec))) = 1;
   
    % sparse representation
    indrow = fix((ind-0.5)/ n) +1;
    indcol = rem(ind, n); 
    indcol = indcol + n*(indcol==0);

    Sigma = sparse(indrow, indcol, Sigmavec); %zsd: max(indrow)=n and max(indcol)=n,since diagonal is not empty...   Sigma = sparse(indrow, indcol, Sigmavec, n, n);
%     Sigma = Sigma .* (~isnan(Sigma)) + 1e-3*eye(n);
else
%     t = sum(Hvec)
    Sigmavec = phi/2^(nu-1)/gamma(nu)* (alpha*Hvec).^nu.* ...
        besselk(nu,alpha*full(Hvec) + decision).* CovTapvec;
    Sigmavec = Sigmavec .* (~isinf(Sigmavec));
    ind_nan = isnan(Sigmavec)==1;
    Sigmavec(ind_nan) =  1;
    Sigma = sparse(Hindi, Hindj, Sigmavec) + 1e-3*eye(n);

end
